######################################################################
#:CQCC::Timer
#
# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2001, 2008 All Rights Reserved
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# Copyright 2001 Rational Software Corporation. All Rights Reserved
# This software contains proprietary and confidential information of
# Rational and its suppliers.  Use, disclosure or reproduction is
# prohibited without the prior express written consent of Rational.

=head1 NAME 

CQCC::Timer - Support class to measure internal run times

=head1 CLASS DESCRIPTION

This class provides a means of measuring elapsed times for important
internal operations to help identify areas for performance tuning. 

It is used through three accessor functions. "TimerStart(label)"
creates a new timer and returns it; "TimerStop(timer)" stops the 
timer; and "TimerLog()" will write out the timing information. 

SUPPORT POLICY: See TriggerCQCC.pm for the Rational Support Policy. 

=head1 METHODS

=cut

######################################################################
#::Package declarations
######################################################################

package CQCC::Timer;
use Exporter;
@ISA = ('Exporter');
@EXPORT = qw(&TimerStart &TimerStop &TimerLog);
             
use strict;
sub Version { return '2.1' };

use CQCC::Parm;
use CQCC::IOBase;

######################################################################
#::Class Globals 
######################################################################

my $seqNumber = 0;           # Each timer is given a new seq number
my $levelNumber = 0;         # Keep track of nestsing levels
my @timers;                  # List of timers

######################################################################
#::function ClassInit

=over 6

=item C<ClassInit()>

This class function is automatically invoked to define class 
parameters using CQCC::Parm objects. Current parameters include: 

=over 4

=item C<CQCC_TIMER>

This parameter enables internal timing data to be recorded for
major operations. It can be set to 0 (off, the default) or 1 (on).

=back

=back

=cut

######################################################################

sub ClassInit { 
    if (&ParmDefined("CQCC_TIMER")) {
        return;
    }     
    &SetParmDefaults(PCLASS=>"CQCC::Timer", SRC_CFG=>1, SRC_EV=>1);

    &DefineParm(NAME=>"CQCC_TIMER", DEFAULT=>0);

    &SetParmDefaults();
}

&ClassInit();

######################################################################
#::function ClassTest

=over 6

=item C<ClassTest(@args)>

This class function creates several timer instances then writes
them out to standard output. 

=back

=cut

######################################################################

sub ClassTest { 
    my @args = @_;
    my ($i,$j);
    my $oldFlag = &GetParm("CQCC_TIMER");
    &ForceParm("CQCC_TIMER",1);

    my $t1 = TimerStart("Loop 10000 times");
    for ($i = 0; $i < 10000; $i++) {
        my $a = 100*100;
    }    
    &TimerStop($t1);

    my $t2 = TimerStart("Outer Loop - 3 times");
    for ($i = 0; $i < 3; $i++) { 
        my $t3 = TimerStart("Loop 100 times");
        for ($j = 0; $j < 100; $j++) {
            my $a = 100*100;
        }
        &TimerStop($t3);
    }    
    &TimerStop($t2);
   
    &TimerLog();
    &ForceParm("CQCC_TIMER", $oldFlag);
    return 0;
}

######################################################################
#::method new

=over 6

=item C<new CQCC::Timer()>

This method initializes the timer's attributes and starts it.

=back

=cut

######################################################################

sub new { 
   my $proto = shift;
   my $class = ref($proto) || $proto;  # Object or class name

   # Attributes
   my $self = {
       CLASSNAME => "CQCC::Timer",
       LABEL => undef,         # Purpose of timer
       SEQNO => undef,         # Sequence number 
       LEVEL => undef,         # Use later to track nesting
       CALLER => undef,        # Name of caller that initiated call
       START => undef,         # Start Times()
       START_SECS => undef,    # Start Time()
       STOP => undef,          # Stop Times()
       STOP_SECS => undef,     # Stop Time()
       @_,                     # Override previous attributes
       @_,                     # Extra time for ccperl
   };

   bless($self, $class);
   my @tlist = ($] ge "5.006") ? times() : (0,0,0,0);
   $self->{LEVEL} = $levelNumber++;
   $self->{START} = \@tlist;
   $self->{START_SECS} = time();
   $self->{SEQNO} = $seqNumber++;
   push @timers, $self;
   return $self;
}

   
######################################################################
#::function TimerLog

=over 6

=item C<TimerLog()>

This function prints the times from the timers if timers are enabled
to standard output and/or the trigger output log file. 

=back

=cut 

######################################################################

sub TimerLog {
    if (&GetParm("CQCC_TIMER") eq "1") { 
        # Note: filename argument is not currently supported
        my $filename = shift; 

        my $line = sprintf "%-35s %10s %10s %10s\n","Label","Secs", 
           "CPU-User","CPU-System";
        &CQCC::IOBase::IOPrint("Timer Log\n", $line);

        if (! defined $filename) {
           my $timer;
           for $timer (@timers) {
               my ($user0, $sys0, $cuser0, $csys0) = 
                   @{ $timer->{START} };
               my $stop = $timer->{STOP};
               my ($user1, $sys1, $cuser1, $csys1) = 
                   (defined $stop) ? @{ $stop } : (0,0,0,0);

               $line = sprintf "%-35.35s %10.3s %10.3s %10.3s\n",
                   substr("               ",0,$timer->{LEVEL}) . 
                      $timer->{LABEL},
                   ((defined $timer->{STOP_SECS}) ? 
                       ($timer->{STOP_SECS} - $timer->{START_SECS}) :
                       "n/a"),
                   ((defined $user1) ? $user1 - $user0 : "n/a"),
                   ((defined $sys1) ? $sys1 - $sys0 : "n/a");
                   &CQCC::IOBase::IOPrint($line);
           }
        }
    }
}

######################################################################
#::function TimerStart

=over 6

=item C<TimerStart(Label)>

This method initializes a new timer, adds it to an internal list, 
then returns it. The caller is expected to call TimerStop(timer) 
later. 

=back

=cut 

######################################################################

sub TimerStart {
    my $label = shift;
    if (&GetParm("CQCC_TIMER") > 0) {
        my ($package, $filename, $line) = caller;
        return new CQCC::Timer(LABEL=>$label, CALLER=>$filename);
    }
    return 0;
}

######################################################################
#::function TimerStop

=over 6

=item C<TimerStop(timer)>

This function takes an existing timer and stops it by storing the
stop time. 

=back

=cut 

######################################################################

sub TimerStop {
    if (&GetParm("CQCC_TIMER") > 0) {
        my $self = shift;
        my @tlist = ($] ge "5.006") ? times() : (0,0,0,0);
        $self->{STOP} = \@tlist;
        $self->{STOP_SECS} = time();
        $levelNumber--;
    }
}

######################################################################
#::Tail
######################################################################

1;
